#ifndef WORKMAN_H
#define WORKMAN_H
/*
 * This file is part of WorkMan, the civilized CD player library
 * Copyright (C) 1991-1997 by Steven Grimm <koreth@midwinter.com>
 * Copyright (C) by Dirk Försterling <milliByte@DeathsDoor.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * all-in-one libworkman include file.
 *
 */


/*
 * wm_config should always be included first
 */
#include "wm_config.h"
#include "workman_defs.h"
#include "wm_cdda.h"
#include "wm_cddb.h"
#include "wm_cdrom.h"
#include "wm_helpers.h"
#include "wm_platform.h"
#include "wm_scsi.h"
#include "wm_struct.h"
#include "wm_cdtext.h"

#endif /* WORKMAN_H */

